@@ -1,6 +1,6 @@
 #!/usr/bin/python
 # encoding=UTF-8
-# Copyright © 2008, 2009 Jakub Wilk <ubanus@users.sf.net>
+# Copyright © 2008, 2009, 2010 Jakub Wilk <ubanus@users.sf.net>
 #
 # This package is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,14 +16,17 @@ from __future__ import with_statement
 import argparse
 import contextlib
 import inspect
+import locale
 import os.path
 import re
 import shutil
 import sys
 import tempfile
+import threading
 
 import djvu.decode
 
+from . import cuneiform
 from . import errors
 from . import hocr
 from . import ipc
@@ -37,6 +40,8 @@ PIXEL_FORMAT = djvu.decode.PixelFormatPackedBits('>')
 PIXEL_FORMAT.rows_top_to_bottom = 1
 PIXEL_FORMAT.y_top_to_bottom = 1
 
+system_encoding = locale.getpreferredencoding()
+
 class Saver(object):
 
     in_place = False
@@ -179,10 +184,46 @@ class Ocropus(OcrEngine):
         finally:
             ocropus.wait()
 
+class Cuneiform(OcrEngine):
+
+    name = 'cuneiform'
+
+    def __init__(self):
+        try:
+            cuneiform.get_languages()
+        except errors.UnknownLanguageList:
+            raise errors.EngineNotFound(self.name)
+
+    @staticmethod
+    def get_default_language():
+        return 'eng'
+
+    @staticmethod
+    def has_language(language):
+        return cuneiform.has_language(language)
+
+    @staticmethod
+    def list_languages():
+        return iter(cuneiform.get_languages())
+
+    def recognize(self, pbm_file, language, details=hocr.TEXT_DETAILS_WORD):
+        return cuneiform.recognize(pbm_file, language)
+
+def get_cpu_count():
+    try:
+        import multiprocessing
+        return multiprocessing.cpu_count()
+    except (ImportError, NotImplementedError):
+        pass
+    try:
+        return os.sysconf('SC_NPROCESSORS_ONLN')
+    except (ValueError, OSError, AttributeError):
+        return 1
+
 class ArgumentParser(argparse.ArgumentParser):
 
     savers = BundledSaver, IndirectSaver, ScriptSaver, InPlaceSaver, DryRunSaver
-    engines = Ocropus,
+    engines = Ocropus, Cuneiform
     default_engine = Ocropus
 
     _details_map = dict(
@@ -215,14 +256,15 @@ class ArgumentParser(argparse.ArgumentParser):
                     help=saver_type.__doc__
                 )
             )
-        self.add_argument('--engine', dest='engine', nargs=1, action=self.set_engine, default=self.default_engine, help=argparse.SUPPRESS or 'set OCR engine')
-        self.add_argument('--list-engines', action=self.list_engines, nargs=0, help=argparse.SUPPRESS or 'print list of available OCR engines')
+        self.add_argument('--engine', dest='engine', nargs=1, action=self.set_engine, default=self.default_engine, help='OCR engine to use')
+        self.add_argument('--list-engines', action=self.list_engines, nargs=0, help='print list of available OCR engines')
         self.add_argument('--ocr-only', dest='ocr_only', action='store_true', default=False, help='''don't save pages without OCR''')
         self.add_argument('--clear-text', dest='clear_text', action='store_true', default=False, help='remove existing hidden text')
         self.add_argument('--language', dest='language', help='set recognition language')
         self.add_argument('--list-languages', action=self.list_languages, nargs=0, help='print list of available languages')
         self.add_argument('-p', '--pages', dest='pages', action='store', default=None, help='pages to process')
         self.add_argument('-D', '--debug', dest='debug', action='store_true', default=False, help='''don't delete intermediate files''')
+        self.add_argument('-j', '--jobs', dest='n_jobs', metavar='N', nargs='?', type=int, default=1, help='number of jobs to run simultaneously')
         self.add_argument('path', metavar='FILE', help='DjVu file to process')
         group = self.add_argument_group(title='text segmentation options')
         group.add_argument('-t', '--details', dest='details', choices=('lines', 'words', 'chars'), action='store', default='words', help='amount of text details to extract')
@@ -307,14 +349,10 @@ class ArgumentParser(argparse.ArgumentParser):
             # For now, let's assume the language pack is installed
             pass
         options.uax29 = options.language if options.word_segmentation == 'uax29' else None
+        if options.n_jobs is None:
+            options.n_jobs = get_cpu_count()
         return options
 
-def validate_file_id(id):
-    if re.compile(r'[/\\\s]').search(id):
-        raise errors.SecurityConcern()
-    return id
-
-
 class Context(djvu.decode.Context):
 
     def init(self, options):
@@ -362,7 +400,8 @@ class Context(djvu.decode.Context):
                     text, = hocr.extract_text(html_file,
                         rotation=page.rotation,
                         details=self._options.details,
-                        uax29=self._options.uax29
+                        uax29=self._options.uax29,
+                        page_size=size
                     )
                     return text
                 finally:
@@ -371,30 +410,77 @@ class Context(djvu.decode.Context):
         finally:
             pfile.close()
 
+    def page_thread(self, pages, results, condition):
+        for page in pages:
+            n = page.n
+            with condition:
+                result = results[n]
+                if result is not None:
+                    # The page is being processed or has been already processed.
+                    continue
+                # Mark the page as taken.
+                results[n] = True
+            try:
+                result = self.process_page(page)
+            except djvu.decode.NotAvailable:
+                print >>sys.stderr, 'No image suitable for OCR.'
+                result = False
+            except Exception, ex:
+                results[n] = ex
+                with condition:
+                    condition.notify()
+                raise
+            with condition:
+                assert results[n] is True
+                results[n] = result
+                condition.notify()
+
     def process(self, path, pages=None):
         try:
             self._engine = self._options.engine()
         except errors.EngineNotFound, ex:
             print >>sys.stderr, ex
             sys.exit(1)
-        print >>sys.stderr, 'Processing %r:' % path
+        print >>sys.stderr, 'Processing %s:' % utils.smart_repr(path, system_encoding)
         document = self.new_document(djvu.decode.FileURI(path))
         document.decoding_job.wait()
         if pages is None:
             pages = list(document.pages)
         else:
             pages = [document.pages[i - 1] for i in pages]
+        results = dict((page.n, None) for page in pages)
+        condition = threading.Condition()
+        threads = [
+            threading.Thread(target=self.page_thread, args=(pages, results, condition))
+            for i in xrange(self._options.n_jobs)
+        ]
+        for thread in threads:
+            thread.start()
         sed_file = self._temp_file('ocrodjvu.djvused')
         try:
             if self._options.clear_text:
                 sed_file.write('remove-txt\n')
             for page in pages:
-                sed_file.write('select %s\n' % validate_file_id(page.file.id))
+                file_id = page.file.id.encode(system_encoding)
+                sed_file.write('select \'%s\'\n' % file_id.replace('\\', '\\\\').replace("'", "\\'"))
                 sed_file.write('set-txt\n')
-                try:
-                    self.process_page(page).print_into(sed_file)
-                except djvu.decode.NotAvailable:
-                    print >>sys.stderr, 'No image suitable for OCR.'
+                with condition:
+                    while 1:
+                        result = results[page.n]
+                        if result is None or result is True:
+                            # Result is not yet available.
+                            condition.wait()
+                        elif isinstance(result, Exception):
+                            for thread in threads:
+                                thread.join()
+                            sys.exit(1)
+                        else:
+                            break
+                if result is False:
+                    # No image suitable for OCR.
+                    pass
+                else:
+                    result.print_into(sed_file)
                 sed_file.write('\n.\n\n')
             sed_file.flush()
             saver = self._options.saver
